Index: channels/chan_dahdi.c
===================================================================
--- channels/chan_dahdi.c	(revision 203126)
+++ channels/chan_dahdi.c	(working copy)
@@ -2245,6 +2245,21 @@
 	return 0;
 }
 
+static int my_set_mwi(void *pvt)
+{
+	struct dahdi_pvt *p = pvt;
+
+	switch (p->sig) {
+		case SIG_FXOLS:
+		case SIG_FXOGS:
+		case SIG_FXOKS:
+			p->onhooktime = time(NULL);
+			p->fxsoffhookstate = 0;
+			p->msgstate = -1;
+	}
+	return 0;
+}
+
 static int dahdi_ring_phone(struct dahdi_pvt *p);
 
 static int my_ring(void *pvt)
@@ -2412,6 +2427,7 @@
 	.wait_event = my_wait_event,
 	.is_off_hook = my_is_off_hook,
 	.set_echocanceller = my_set_echocanceller,
+ 	.set_mwi = my_set_mwi,
 	.ring = my_ring,
 	.off_hook = my_off_hook,
 	.dial_digits = my_dial_digits,
Index: channels/sig_analog.c
===================================================================
--- channels/sig_analog.c	(revision 203123)
+++ channels/sig_analog.c	(working copy)
@@ -372,6 +372,14 @@
 		return -1;
 }
 
+static int analog_set_mwi(struct analog_pvt *p)
+{
+	if (p->calls->set_mwi)
+		return p->calls->set_mwi(p->chan_pvt);
+	else
+		return -1;
+}
+
 static int analog_train_echocanceller(struct analog_pvt *p)
 {
 	if (p->calls->train_echocanceller)
@@ -2395,6 +2403,7 @@
 			/* Fall through */
 		default:
 			analog_set_echocanceller(p, 0);
+			analog_set_mwi(p);
 			return NULL;
 		}
 		break;
@@ -3144,6 +3153,7 @@
 			analog_set_echocanceller(i, 0);
 			res = analog_play_tone(i, ANALOG_SUB_REAL, -1);
 			analog_on_hook(i);
+			analog_set_mwi(i);
 			break;
 		case ANALOG_SIG_FXOKS:
 			analog_set_echocanceller(i, 0);
@@ -3154,6 +3164,7 @@
 #endif
 			res = analog_play_tone(i, ANALOG_SUB_REAL, -1);
 			analog_on_hook(i);
+			analog_set_mwi(i);
 			break;
 		default:
 			ast_log(LOG_WARNING, "Don't know how to handle on hook with signalling %s on channel %d\n", analog_sigtype_to_str(i->sig), i->channel);
Index: channels/sig_analog.h
===================================================================
--- channels/sig_analog.h	(revision 203123)
+++ channels/sig_analog.h	(working copy)
@@ -182,6 +182,8 @@
 	int (* const check_for_conference)(void *pvt);
 	void (* const handle_notify_message)(struct ast_channel *chan, void *pvt, int cid_flags, int neon_mwievent);
 
+	/* Stimulus to cause mwi processing on the channel */
+	int (* const set_mwi)(void *pvt);
 	/* callbacks for increasing and decreasing ss_thread_count, will handle locking and condition signal */
 	void (* const increase_ss_count)(void);
 	void (* const decrease_ss_count)(void);
